/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Laboratorul 1
 * Digital I/O
 *********************************************************************************/

#include <avr\io.h>
#include <avr\interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "lcd.h"

#define IDLE 0
#define PROMPT 1
#define MEASURE 2
#define SHOW_RES 3

void init_adc(){
 
 	ADMUX = 0;
	ADCSRA = 0;
	ADCSRA |= ((1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0));    //16Mhz/128 = 125Khz the ADC reference clock
	ADMUX |= (1 << REFS0) | (1 << ADLAR) | (1 << MUX2)| (1 << MUX1)| (1 << MUX0) ;
	ADCSRA |= (1 << ADEN);  // Enable ADC
}

int read_adc()
{
	int value = 0;

	ADMUX &= ~( (1 << MUX3) | ( 1 << MUX4) );
	ADMUX |= (1 << MUX2)| (1 << MUX1)| (1 << MUX0);
	ADCSRA |= (1 << ADSC);
	
	while( !(ADCSRA & ( 1 << ADIF )) );
	value = ADC;

	return value;
}

int main() {
	LCD_init();

	DDRD = 0xFF;

	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6);



	char message_idle[] = "  Apasati butonul pentru a incepe";
	char message_prep[] = "  Suflati puternic timp de 4-5 secunde";
	char base_message[] = "Am detectat ";
	char buf[20];

	int state; // 0 - Idle
			   // 1 - Prompt
			   // 2 - Measure
               // 3 - Show results

	state = 0;
	LCD_print2(message_idle, message_idle + 8);
	_delay_ms(250);

	int world_has_ended = 0, state_has_changed;
	init_adc();
	while (!world_has_ended) {
		
		state_has_changed = 0;

		LCD_writeInstruction(0x18);
		_delay_ms(250);

		if ((PIND & (1 << PD6)) == 0) {
			state = (state + 1) % 4;
			state_has_changed = 1;

			
		}

	
		if (state_has_changed) {
			switch (state) {
				LCD_writeInstruction(LCD_INSTR_clearDisplay);
				case (IDLE):
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					LCD_print2(message_idle, message_idle + 8);
					_delay_ms(500);
					break;

				case (PROMPT):
					LCD_print2(message_prep, message_prep + 8);
					_delay_ms(500);
					break;

				case (SHOW_RES):
					
					sprintf(buf, "%s %d!", base_message, read_adc());

					LCD_print2(buf, buf + 8);
					_delay_ms(500);
					break;

				default:
					;
			}
		}
	
	}

	return 0;
}
